package com.dsic.gj_erp.util;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.dsic.gj_erp.bean.jhgk.DmJtsyjh;
import com.dsic.gj_erp.bean.jhgk.DmXbjh;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.time.DateUtils;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.usermodel.*;
import org.apache.poi.ss.util.CellRangeAddress;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.io.InputStream;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class ExcelUtil {
    public static Map<String,String> ZRCJHKeyMap=new HashMap<String,String>(){
        {
            //put("状态","zt");put("周期（天）","b");
            put("船号","dcCh");
            put("区域","qy");
            put("总段号","zdh");
            put("预组段","yzd");
            put("分段号","dcFd");
            put("重量","zl");
            put("批量号","dcPl");
            put("大组开始","kssjDz");
            put("钢加结束大组","gjjsDz");
            put("小组结束","jssjXz");
            put("小组周期","xzzq");
            put("小组开始","kssjXz");
            put("线表重量","xbzl");
            put("批量表（钢加大组结束）","gjjsDzPl");
            put("时间差（L-P）","sjcLp");
            put("钢加小组结束","gjjsXz");
            put("批量表（钢加小组结束）","gjjsXzPl");
            put("时间差（R-S）","sjcRs");
            put("钢加开始","gjks");
            put("批量表（钢加开始）","gjksPl");
            put("时间差（U-V）","sjcUv");
            put("套料图下发","xfqTlt");
            put("套料图接收","jsqTlt");
            put("板材领取表下发","xfqLqb");
            put("板材领取表接收","jsqLqb");
            put("零件配套表下发","xfqPtb");
            put("零件配套表接收","jsqPtb");
            put("订货清单号","dhqd");
            put("订货清单下发","xfqDhqd");
            put("实际订货日期","sjdhq");
            put("板材计划到齐","bcdq");
        }
    };

    public static List<String> ZRCJHYearKeyList=Arrays.asList(
            "大组开始","钢加结束大组","小组结束","小组开始","钢加小组结束","批量表（钢加大组结束）",
            "钢加开始","批量表（钢加开始）","套料图下发","板材领取表下发","订货清单下发","实际订货日期","板材计划到齐"
    );

    public static Map<String,String> JTSYJHKeyMap=new HashMap<String,String>(){
        {
            put("船名","dcCh");put("批量","dcPl");put("分段","dcFd"); put("计划期","jhrq");put("备注","bz");
        }
    };
//
//    public static List<String> JTSYJHHYearKeyList=Arrays.asList(
//            "计划期"
//    );

    public static  List<Map<String,Object>> readJTSYJH(MultipartFile file) {
        try {
            Workbook workbook = isWorkBook(file);
            // 获取第一张表
            Sheet sheet = workbook.getSheetAt(0);
            // sheet.getPhysicalNumberOfRows()获取总的行数
            // 循环读取每一行
            Row headerRow = sheet.getRow(1);
            Map<Integer,String> headerMap=new HashMap<>();
            for (int index = 0; index < headerRow.getPhysicalNumberOfCells(); index++){
                headerMap.put(index,getMergedRegionValue(sheet,1,index)+"");
            }

            List<Map<String,Object>> mapList=new ArrayList<>();
            for (int i = 3; i < sheet.getPhysicalNumberOfRows(); i++) {
                // 循环读取每一个格
                Row row = sheet.getRow(i);
                // row.getPhysicalNumberOfCells()获取总的列数
                String dcch=row.getCell(0).toString();
                if (dcch.contains("合计")){
                    continue;
                }
                Map<String,Object> dataMap=new HashMap<>();
                for (int index = 0; index < row.getPhysicalNumberOfCells(); index++) {
                    String key=headerMap.get(index);
                    String mapKey=JTSYJHKeyMap.get(key);
                    if (mapKey==null){
                        continue;
                    }
                    Cell cell;
                    if (isMergedRow(sheet,i,index)){
                        cell=getMergedRegionValue(sheet,i,index);
                    }
                    else {
                        cell = row.getCell(index);
                        cell.setCellType(CellType.STRING);
                    }
                    String value=cell+"";
                    if (key.equals("计划期")){
                        if (StringUtils.isEmpty(value)){
                            continue;
                        }
                        else {
                            value=date(value);
                        }
                    }
                    if (key.equals("批量")){
                        if (StringUtils.isEmpty(value)){
                            continue;
                        }
                        else {
                            value=String.format("%03d",Integer.parseInt(value));
                        }
                    }

                    dataMap.put(mapKey,value);
                }
                mapList.add(dataMap);
            }
            return  mapList;
        } catch (Exception e) {
            e.printStackTrace();
            return null;
        }
    }

    public static List<Map<String,Object>> readZRCJH(MultipartFile file) {
        try {
            List<Map<String,Object>> list=new ArrayList<>();
            Workbook workbook = isWorkBook(file);
            // 获取第一张表
            Sheet sheet = workbook.getSheetAt(0);
            Row headerRow = sheet.getRow(0);
            Map<Integer,String> headerMap=new HashMap<>();
            for (int index = 0; index < headerRow.getPhysicalNumberOfCells(); index++){
                Cell cell = headerRow.getCell(index);
                // 转换为字符串类型
                cell.setCellType(CellType.STRING);
                headerMap.put(index,cell.toString().replace("\n",""));
            }

            // 循环读取每一行
            for (int i = 1; i < sheet.getPhysicalNumberOfRows(); i++) {
                Map<String,Object> dataMap=new HashMap<>();
                // 循环读取每一个格
                Row row = sheet.getRow(i);
                // row.getPhysicalNumberOfCells()获取总的列数
                for (int index = 0; index < row.getPhysicalNumberOfCells(); index++) {
                    String key=headerMap.get(index);
                    String mapKey=ZRCJHKeyMap.get(key);
                    if (mapKey==null){
                        continue;
                    }
                    Cell cell;
                    if (isMergedRow(sheet,i,index)){
                        cell=getMergedRegionValue(sheet,i,index);
                    }
                    else {
                        cell = row.getCell(index);
                        cell.setCellType(CellType.STRING);
                    }
                    String value=cell+"";
                    if (ZRCJHYearKeyList.contains(key)){
                        value=date(value);
                    }
                    dataMap.put(mapKey,value);
                }
                list.add(dataMap);
            }
            return list;
        } catch (Exception e) {
            e.printStackTrace();
            return null;
        }
    }

    public static JSONObject readDmXbjh(MultipartFile file) {
        try {
            Workbook workbook=isWorkBook(file);
            // 获取第一张表
            Sheet sheet = workbook.getSheetAt(0);
            Row monthRow=sheet.getRow(2);
            Map<Integer,String> monthMap=new HashMap<>();
            for (int index = 0; index < monthRow.getPhysicalNumberOfCells(); index++){
                Cell cell = monthRow.getCell(index);
                // 转换为字符串类型
                cell.setCellType(CellType.STRING);
                monthMap.put(index,cell.toString());
            }
           // String year="";
          //  List<Map<String,Object>> mapList=new ArrayList<>();
           // List<DmXbjh> dmXbjhs=new ArrayList<>();
            JSONObject jsonObject=new JSONObject();
            for (int i = 3; i < sheet.getPhysicalNumberOfRows(); i++) {
                Row row = sheet.getRow(i);
                String dcch=row.getCell(0).toString();
                if (dcch.contains("待接")||dcch.contains("已接")||dcch.contains("合计")){
                    break;
                }
//                Map<String,Object> map=new HashMap<>();
//                map.put("dcCh",dcch);
                List<Map> mapList=new ArrayList<>();
                for (int index = 1; index < row.getPhysicalNumberOfCells(); index++) {
                    Cell cell = row.getCell(index);
                    cell.setCellType(CellType.STRING);

                    String month=monthMap.get(index);
                    if (month.contains("计")){
                        continue;
                    }
                    String cl=cell+"";
                    String year=getMergedRegionValue(sheet,1,index).toString();
                 //   DmXbjh dmXbjh=new DmXbjh();
                    Map map=new HashMap();
                    map.put("zt","0");
                    map.put("nf",year.replace("年",""));
                    map.put("yf",String.format("%02d", Integer.parseInt(month.replace("月",""))));
                    if (StringUtils.isNotEmpty(cl)){
                        map.put("zl",Double.parseDouble(cl));
                    }else {
                        map.put("zl",0.0);
                    }
//                    dmXbjh.setZt("0");
//                    dmXbjh.setNf(year.replace("年",""));
//                    dmXbjh.setYf(String.format("%02d", Integer.parseInt(month.replace("月",""))));
//                    if (StringUtils.isNotEmpty(cl)){
//                        dmXbjh.setZl(Double.parseDouble(cl));
//                    }else {
//                        dmXbjh.setZl(0.0);
//                    }
//                    dmXbjh.setDcCh(dcch);


               //     dmXbjhs.add(dmXbjh);
                    mapList.add(map);
//                    map.put("cl"+year.replace("年","")+String.format("%02d", Integer.parseInt(month.replace("月",""))),cl);
                }
//                mapList.add(map);
                jsonObject.put(dcch,mapList);
            }
            return jsonObject;
//            return mapList;
          //  return dmXbjhs;
        } catch (Exception e) {
            e.printStackTrace();
            return null;
        }
    }

    public static String date(String strDate){
        try {
            Calendar calendar = new GregorianCalendar(1900,0,-1);
            int intDay = Integer.parseInt(strDate);
            Date dd = DateUtils.addDays(calendar.getTime(),intDay);
            SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy/MM/dd");
            String format = simpleDateFormat.format(dd);
            return format;
        }catch (Exception e){
            return "";
        }
    }

    public static boolean isMergedRow(Sheet sheet, int row, int column) {
        int sheetMergeCount = sheet.getNumMergedRegions();
        for (int i = 0; i < sheetMergeCount; i++) {
            CellRangeAddress range = sheet.getMergedRegion(i);
            int firstCol = range.getFirstColumn();
            int lastCol = range.getLastColumn();
            int firstRow = range.getFirstRow();
            int lastRow = range.getLastRow();
            if (row >= firstRow && row <= lastRow) {
                if (column >= firstCol && column <= lastCol) {
                    return true;
                }
            }
        }
        return false;
    }

    public static Workbook isWorkBook(MultipartFile file) throws IOException {
        Workbook workbook;// = new XSSFWorkbook(inputStream);
        // 截取路径名 . 后面的后缀名，判断是xls还是xlsx
        // 如果这个判断不对，就把equals换成 equalsIgnoreCase()
        Pattern pattern = Pattern.compile(".*\\.xls");
        Matcher matcher = pattern.matcher(file.getOriginalFilename());
        if (matcher.matches()){
            workbook = new HSSFWorkbook(file.getInputStream());
        }else{
            workbook = new XSSFWorkbook(file.getInputStream());
        }
        return workbook;
    }
    /**
     * 获取合并单元格的值
     * @param sheet
     * @param row
     * @param column
     * @return
     */
    public static Cell getMergedRegionValue(Sheet sheet ,int row , int column){
        int sheetMergeCount = sheet.getNumMergedRegions();

        for(int i = 0 ; i < sheetMergeCount ; i++){
            CellRangeAddress ca = sheet.getMergedRegion(i);
            int firstColumn = ca.getFirstColumn();
            int lastColumn = ca.getLastColumn();
            int firstRow = ca.getFirstRow();
            int lastRow = ca.getLastRow();

            if(row >= firstRow && row <= lastRow){

                if(column >= firstColumn && column <= lastColumn){
                    Row fRow = sheet.getRow(firstRow);
                    Cell fCell = fRow.getCell(firstColumn);
                    return fCell ;
                }
            }
        }

        return null ;
    }

}
